@@ -1,20 +1,11 @@
+from sympy.external import import_module
 from spb.defaults import cfg
 from spb.backends.base_backend import Plot
-from spb.backends.utils import convert_colormap
-from spb.utils import get_vertices_indices
 from spb.backends.utils import compute_streamtubes
-import k3d
-import numpy as np
+from spb.utils import get_vertices_indices
 import warnings
-from matplotlib.tri import Triangulation
-from mergedeep import merge
-import colorcet as cc
 import os
 
-# TODO:
-# 1. load the plot with menu minimized
-# 2. iplot support for streamlines?
-
 
 class K3DBackend(Plot):
     """A backend for plotting SymPy's symbolic expressions using K3D-Jupyter.
@@ -25,13 +16,13 @@ class K3DBackend(Plot):
     line_kw : dict, optional
         A dictionary of keywords/values which is passed to K3D's line
         functions to customize the appearance. Default to:
-        ``line_kw = dict(width=0.1, shader="mesh")``
-        Set ``use_cm=False`` to switch to a solid color.
+        ``line_kw = dict(width=0.1, shader="mesh")``.
+        Set `use_cm=False` to switch to a solid color.
 
     quiver_kw : dict, optional
         A dictionary to customize the apppearance of quivers. Default to:
         ``quiver_kw = dict(scale = 1)``.
-        Set ``use_cm=False`` to switch to a solid color.
+        Set `use_cm=False` to switch to a solid color.
 
     show_label : boolean, optional
         Show/hide labels of the expressions. Default to False (labels not
@@ -40,9 +31,9 @@ class K3DBackend(Plot):
     stream_kw : dict, optional
         A dictionary to customize the apppearance of streamlines.
         Default to:
-        ``stream_kw = dict( width=0.1, shader='mesh' )``
+        ``stream_kw = dict( width=0.1, shader='mesh' )``.
         Refer to k3d.line for more options.
-        Set ``use_cm=False`` to switch to a solid color.
+        Set `use_cm=False` to switch to a solid color.
 
     use_cm : boolean, optional
         If True, apply a color map to the meshes/surface. If False, solid
@@ -57,35 +48,51 @@ class K3DBackend(Plot):
 
     _library = "k3d"
 
-    colormaps = [
-        k3d.basic_color_maps.CoolWarm,
-        k3d.matplotlib_color_maps.Plasma,
-        k3d.matplotlib_color_maps.Winter,
-        k3d.matplotlib_color_maps.Viridis,
-        k3d.paraview_color_maps.Haze,
-        k3d.matplotlib_color_maps.Summer,
-        k3d.paraview_color_maps.Blue_to_Yellow,
-    ]
-
-    cyclic_colormaps = [cc.colorwheel, k3d.paraview_color_maps.Erdc_iceFire_H]
+    colormaps = []
+    cyclic_colormaps = []
 
     def __new__(cls, *args, **kwargs):
-        # Since Plot has its __new__ method, this will prevent infinite
-        # recursion
         return object.__new__(cls)
 
     def __init__(self, *args, **kwargs):
+        self.k3d = k3d = import_module(
+            'k3d',
+            import_kwargs={'fromlist': ['helpers', 'objects']},
+            min_module_version='2.9.7')
+        cc = import_module(
+            'colorcet',
+            min_module_version='3.0.0')
+        self.matplotlib = import_module(
+            'matplotlib',
+            import_kwargs={'fromlist': ['tri', 'cm']},
+            min_module_version='1.1.0',
+            catch=(RuntimeError,))
+        cm = self.matplotlib.cm
+
+        self.colorloop = cm.tab10.colors
+        self.colormaps = [
+            k3d.basic_color_maps.CoolWarm,
+            k3d.matplotlib_color_maps.Plasma,
+            k3d.matplotlib_color_maps.Winter,
+            k3d.matplotlib_color_maps.Viridis,
+            k3d.paraview_color_maps.Haze,
+            k3d.matplotlib_color_maps.Summer,
+            k3d.paraview_color_maps.Blue_to_Yellow,
+        ]
+        self.cyclic_colormaps = [
+            cc.colorwheel, k3d.paraview_color_maps.Erdc_iceFire_H]
+
+        self._init_cyclers()
         super().__init__(*args, **kwargs)
         if self._get_mode() != 0:
             raise ValueError(
                 "Sorry, K3D backend only works within Jupyter Notebook")
 
+        self._show_label = kwargs.get("show_label", False)
         self._bounds = []
         self._clipping = []
         self._handles = dict()
 
-        self._init_cyclers()
-
         self._fig = k3d.plot(
             grid_visible=self.grid,
             menu_visibility=True,
@@ -99,12 +106,41 @@ class K3DBackend(Plot):
                 + "continue with linear scales."
             )
         self.plot_shown = False
+
+    @property
+    def fig(self):
+        """Returns the figure."""
+        # K3D title is an object in the figure
+        n = len(self._fig.objects)
+        if self.title is not None:
+            n -= 1
+        if len(self.series) != n:
+            # if the backend was created without showing it
+            self.process_series()
+        return self._fig
+
+    def process_series(self):
+        """ Loop over data series, generates numerical data and add it to the
+        figure.
+        """
+        # this is necessary in order for the series to be added even if
+        # show=False
         self._process_series(self._series)
 
+    def _set_piecewise_color(self, s, color):
+        """Set the color to the given series"""
+        raise NotImplementedError
+
+    @staticmethod
+    def _do_sum_kwargs(p1, p2):
+        kw = p1._copy_kwargs()
+        kw["show_label"] = (p1._show_label or p2._show_label)
+        return kw
+
     @staticmethod
     def _int_to_rgb(RGBint):
-        """Convert an integer number to an RGB tuple with components from 0 to
-        255.
+        """Convert an integer number to an RGB tuple with components from 0
+        to 255.
 
         https://stackoverflow.com/a/2262152/2329968
         """
@@ -124,13 +160,16 @@ class K3DBackend(Plot):
 
     @classmethod
     def _convert_to_int(cls, color):
-        """Convert the provided RGB tuple with values from 0 to 1 to an integer
-        number.
+        """Convert the provided RGB tuple with values from 0 to 1 to an
+        integer number.
         """
         color = [int(c * 255) for c in color]
         return cls._rgb_to_int(color)
 
     def _process_series(self, series):
+        np = import_module('numpy')
+        merge = self.merge
+        Triangulation = self.matplotlib.tri.Triangulation
         self._init_cyclers()
         self._fig.auto_rendering = False
         # clear data
@@ -142,8 +181,8 @@ class K3DBackend(Plot):
                 x, y, z, _ = s.get_data()
                 positions = np.vstack([x, y, z]).T.astype(np.float32)
                 a = dict(point_size=0.2, color=self._convert_to_int(next(self._cl)))
-                line_kw = self._kwargs.get("line_kw", dict())
-                plt_points = k3d.points(positions=positions, **merge({}, a, line_kw))
+                kw = merge({}, a, s.rendering_kw)
+                plt_points = self.k3d.points(positions=positions, **kw)
                 plt_points.shader = "mesh"
                 self._fig += plt_points
 
@@ -158,7 +197,7 @@ class K3DBackend(Plot):
                 # keyword arguments for the line object
                 a = dict(
                     width=0.1,
-                    name=s.label if self._kwargs.get("show_label", False) else None,
+                    name=s.label if self._show_label else None,
                     color=self._convert_to_int(next(self._cl)),
                     shader="mesh",
                 )
@@ -166,8 +205,8 @@ class K3DBackend(Plot):
                     a["attribute"] = (param.astype(np.float32),)
                     a["color_map"] = next(self._cm)
                     a["color_range"] = [s.start, s.end]
-                line_kw = self._kwargs.get("line_kw", dict())
-                line = k3d.line(vertices, **merge({}, a, line_kw))
+                kw = merge({}, a, s.rendering_kw)
+                line = self.k3d.line(vertices, **kw)
                 self._fig += line
 
             elif (s.is_3Dsurface and not s.is_domain_coloring):
@@ -192,7 +231,7 @@ class K3DBackend(Plot):
 
                 self._high_aspect_ratio(x, y, z)
                 a = dict(
-                    name=s.label if self._kwargs.get("show_label", False) else None,
+                    name=s.label if self._show_label else None,
                     side="double",
                     flat_shading=False,
                     wireframe=False,
@@ -201,8 +240,8 @@ class K3DBackend(Plot):
                 if self._use_cm:
                     a["color_map"] = next(self._cm)
                     a["attribute"] = z.astype(np.float32)
-                surface_kw = self._kwargs.get("surface_kw", dict())
-                surf = k3d.mesh(vertices, indices, **merge({}, a, surface_kw))
+                kw = merge({}, a, s.rendering_kw)
+                surf = self.k3d.mesh(vertices, indices, **kw)
 
                 self._fig += surf
 
@@ -214,9 +253,9 @@ class K3DBackend(Plot):
                     else t for t in [xx, yy, zz, uu, vv, ww]]
 
                 vertices, magn = compute_streamtubes(
-                    xx, yy, zz, uu, vv, ww, self._kwargs)
+                    xx, yy, zz, uu, vv, ww, s.rendering_kw)
 
-                stream_kw = self._kwargs.get("stream_kw", dict())
+                stream_kw = s.rendering_kw.copy()
                 skw = dict(width=0.1, shader="mesh")
                 if self._use_cm and ("color" not in stream_kw.keys()):
                     skw["color_map"] = next(self._cm)
@@ -228,8 +267,9 @@ class K3DBackend(Plot):
                         col = self._convert_to_int(col)
                     stream_kw["color"] = col
 
-                self._fig += k3d.line(
-                    vertices.astype(np.float32), **merge({}, skw, stream_kw))
+                kw = merge({}, skw, stream_kw)
+                self._fig += self.k3d.line(
+                    vertices.astype(np.float32), **kw)
 
             elif s.is_3Dvector:
                 xx, yy, zz, uu, vv, ww = s.get_data()
@@ -241,18 +281,17 @@ class K3DBackend(Plot):
                 xx, yy, zz, uu, vv, ww = [
                     t.flatten().astype(np.float32) for t in [xx, yy, zz, uu, vv, ww]
                 ]
-                # default values
                 qkw = dict(scale=1)
-                # user provided values
-                quiver_kw = self._kwargs.get("quiver_kw", dict())
-                qkw = merge(qkw, quiver_kw)
+                qkw = merge(qkw, s.rendering_kw)
                 scale = qkw["scale"]
                 magnitude = np.sqrt(uu ** 2 + vv ** 2 + ww ** 2)
                 vectors = np.array((uu, vv, ww)).T * scale
                 origins = np.array((xx, yy, zz)).T
+                quiver_kw = s.rendering_kw
                 if self._use_cm and ("color" not in quiver_kw.keys()):
                     colormap = next(self._cm)
-                    colors = k3d.helpers.map_colors(magnitude, colormap, [])
+                    colors = self.k3d.helpers.map_colors(
+                        magnitude, colormap, [])
                     self._handles[ii] = [qkw, colormap]
                 else:
                     col = quiver_kw.get("color", next(self._cl))
@@ -265,11 +304,17 @@ class K3DBackend(Plot):
                     vec_colors[2 * i] = c
                     vec_colors[2 * i + 1] = c
                 vec_colors = vec_colors.astype(np.uint32)
-                vec = k3d.vectors(
-                    origins=origins - vectors / 2,
-                    vectors=vectors,
-                    colors=vec_colors,
-                )
+
+                vec_kw = qkw.copy()
+                kw_to_remove = ["scale", "color"]
+                for k in kw_to_remove:
+                    if k in vec_kw.keys():
+                        vec_kw.pop(k)
+                vec_kw["origins"] = origins - vectors / 2
+                vec_kw["vectors"] = vectors
+                vec_kw["colors"] = vec_colors
+
+                vec = self.k3d.vectors(**vec_kw)
                 self._fig += vec
 
             elif s.is_complex and s.is_3Dsurface:
@@ -281,7 +326,7 @@ class K3DBackend(Plot):
                 self._high_aspect_ratio(x, y, z)
 
                 a = dict(
-                    name=s.label if self._kwargs.get("show_label", False) else None,
+                    name=s.label if self._show_label else None,
                     side="double",
                     flat_shading=False,
                     wireframe=False,
@@ -301,8 +346,8 @@ class K3DBackend(Plot):
 
                     a["color_map"] = r
                     a["color_range"] = [-np.pi, np.pi]
-                surface_kw = self._kwargs.get("surface_kw", dict())
-                surf = k3d.mesh(vertices, indices, **merge({}, a, surface_kw))
+                kw = merge({}, a, s.rendering_kw)
+                surf = self.k3d.mesh(vertices, indices, **kw)
 
                 self._fig += surf
 
@@ -318,15 +363,15 @@ class K3DBackend(Plot):
         self._fig.axes = [xl, yl, zl]
 
         if self.title:
-            self._fig += k3d.text2d(
+            self._fig += self.k3d.text2d(
                 self.title, position=[0.025, 0.015], color=0, size=1, label_box=False
             )
         self._fig.auto_rendering = True
 
     def _high_aspect_ratio(self, x, y, z):
         """Look for high aspect ratio meshes, where (dz >> dx, dy) and
-        eventually set the bounds around the mid point of the mesh in order to
-        improve visibility. Bounds will be used to set the camera position.
+        eventually set the bounds around the mid point of the mesh in order
+        to improve visibility. Bounds will be used to set the camera position.
         """
         mz, Mz, meanz = z.min(), z.max(), z.mean()
         mx, Mx = x.min(), x.max()
@@ -335,18 +380,29 @@ class K3DBackend(Plot):
 
         # thresholds
         t1, t2 = 10, 3
-        if (dz / dx >= t1) and (dz / dy >= t1):
+        if (dz / dx >= t1) and (dz / dy >= t1) and (self.zlim is None):
             if abs(Mz / meanz) > t1:
                 Mz = meanz + t2 * max(dx, dy)
             if abs(mz / meanz) > t1:
                 mz = meanz - t2 * max(dx, dy)
             self._bounds.append([mx, Mx, my, My, mz, Mz])
+        elif self.zlim:
+            self._bounds.append([mx, Mx, my, My, self.zlim[0], self.zlim[1]])
 
     def _update_interactive(self, params):
+        np = import_module('numpy')
+
+        # K3D title is an object in the figure
+        n = len(self._fig.objects)
+        if self.title is not None:
+            n -= 1
+        if len(self.series) != n:
+            self._process_series(self.series)
+
         # self._fig.auto_rendering = False
         for i, s in enumerate(self.series):
             if s.is_interactive:
-                self.series[i].update_data(params)
+                self.series[i].params = params
 
                 if s.is_3Dline and s.is_point:
                     x, y, z, _ = self.series[i].get_data()
@@ -377,7 +433,8 @@ class K3DBackend(Plot):
                     magnitude = np.sqrt(uu ** 2 + vv ** 2 + ww ** 2)
                     qkw, colormap = self._handles[i]
                     if colormap is not None:
-                        colors = k3d.helpers.map_colors(magnitude, colormap, [])
+                        colors = self.k3d.helpers.map_colors(
+                            magnitude, colormap, [])
                         vec_colors = np.zeros(2 * len(colors))
                         for j, c in enumerate(colors):
                             vec_colors[2 * j] = c
@@ -403,14 +460,16 @@ class K3DBackend(Plot):
 
     def show(self):
         """Visualize the plot on the screen."""
+        np = import_module('numpy')
 
         if len(self._fig.objects) != len(self.series):
             self._process_series(self._series)
         self.plot_shown = True
 
         if len(self._bounds) > 0:
-            # there are very high aspect ratio meshes. We are going to compute a
-            # new camera position to improve user experience
+            # when there are very high aspect ratio meshes, or when zlim has
+            # been set, we compute a new camera position to improve user
+            # experience
             self._fig.camera_auto_fit = False
             bounds = np.array(self._bounds)
             bounds = np.dstack(
@@ -426,20 +485,20 @@ class K3DBackend(Plot):
             ]
 
     def save(self, path, **kwargs):
-        """ Export the plot to a static picture or to an interactive html file.
+        """Export the plot to a static picture or to an interactive html file.
 
         Notes
         =====
 
         K3D-Jupyter is only capable of exporting:
 
-        1. '.png' pictures: refer to [#fn1]_ to visualize the available keyword
-           arguments.
-        2. '.html' files: when exporting a fully portable html file, by default
-           the required Javascript libraries will be loaded with a CDN. Set
-           ``include_js=True`` to include all the javascript code in the html
-           file: this will create a bigger file size, but can be run without
-           internet connection.
+        1. '.png' pictures: refer to [#fn1]_ to visualize the available
+           keyword arguments.
+        2. '.html' files: when exporting a fully portable html file, by
+           default the required Javascript libraries will be loaded with a
+           CDN. Set `include_js=True` to include all the javascript code in
+           the html file: this will create a bigger file size, but can be
+           run without internet connection.
 
         References
         ==========
