@@ -1,18 +1,23 @@
-import numpy as np
-import param
-import panel as pn
-from sympy import latex, Tuple
+from sympy.printing.latex import latex
+from sympy.core.containers import Tuple
+from sympy.external import import_module
 from spb.series import (
     InteractiveSeries,
-    _set_discretization_points
+    _set_discretization_points,
 )
 from spb.ccomplex.complex import _build_series as _build_complex_series
 from spb.vectors import _preprocess, _build_series as _build_vector_series
 from spb.utils import _plot_sympify, _unpack_args
-from spb.defaults import TWO_D_B, THREE_D_B
-from bokeh.models.formatters import TickFormatter
+from spb.defaults import TWO_D_B, THREE_D_B, cfg
 import warnings
 
+param = import_module(
+    'param',
+    min_module_version='1.11.0')
+pn = import_module(
+    'panel',
+    min_module_version='0.12.0')
+
 pn.extension("plotly")
 
 
@@ -20,11 +25,11 @@ class MyList(param.ObjectSelector):
     """Represent a list of numbers discretizing a log-spaced slider.
     This parameter will be rendered by pn.widgets.DiscreteSlider
     """
-
     pass
 
 
-# explicitely ask panel to use DiscreteSlider when it encounters a MyList object
+# explicitely ask panel to use DiscreteSlider when it encounters a
+# MyList object
 pn.Param._mapping[MyList] = pn.widgets.DiscreteSlider
 
 
@@ -35,10 +40,10 @@ class DynamicParam(param.Parameterized):
 
     # NOTE: why DynamicParam is a child class of param.Parameterized?
     # param is a full-python library, doesn't depend on anything else.
-    # In theory, by using a parameterized class it should be possible to create
-    # an InteractivePlotGUI class targeting a specific GUI.
+    # In theory, by using a parameterized class it should be possible to
+    # create an InteractivePlotGUI class targeting a specific GUI.
     # At this moment, InteractivePlot is built on top of 'panel', so it only
-    # works inside a Jupyter Notebook. Maybe it's possible to use PyQt or Tk...
+    # works inside a Jupyter Notebook. Maybe it's possible to use PyQt or Tk.
 
     def _tuple_to_dict(self, k, v):
         """The user can provide a variable length tuple/list containing:
@@ -68,6 +73,8 @@ class DynamicParam(param.Parameterized):
                 Discretization spacing. Can be "linear" or "log".
                 Default to "linear".
         """
+        np = import_module('numpy')
+
         if len(v) >= 5:
             # remove tick_format, as it won't be used for the creation of the
             # parameter. Its value has already been stored.
@@ -81,7 +88,7 @@ class DynamicParam(param.Parameterized):
             0,
             2,
             N,
-            "$%s$" % latex(k) if self.use_latex else str(k),
+            "$%s$" % latex(k) if self._use_latex else str(k),
             "linear",
         ]
         values = defaults_values.copy()
@@ -95,8 +102,8 @@ class DynamicParam(param.Parameterized):
             values[3] = 1
 
         if values[-1] == "log":
-            # In case of a logarithm slider, we need to instantiate the custom
-            # parameter MyList.
+            # In case of a logarithm slider, we need to instantiate the
+            # custom parameter MyList.
 
             # # divide the range in N steps evenly spaced in a log scale
             options = np.geomspace(_min, _max, N)
@@ -116,6 +123,12 @@ class DynamicParam(param.Parameterized):
         return {k: v for k, v in zip(defaults_keys, values)}
 
     def __init__(self, *args, name="", params=None, **kwargs):
+        bokeh = import_module(
+            'bokeh',
+            import_kwargs={'fromlist': ['models']},
+            min_module_version='2.3.0')
+        TickFormatter = bokeh.models.formatters.TickFormatter
+
         # remove the previous class attributes added by the previous instances
         cls_name = type(self).__name__
         setattr(type(self), "_" + cls_name + "__params", dict())
@@ -124,12 +137,13 @@ class DynamicParam(param.Parameterized):
             delattr(type(self), p)
 
         # use latex on control labels and legends
-        self.use_latex = kwargs.pop("use_latex", True)
+        self._use_latex = kwargs.pop("use_latex", True)
 
         # this must be present in order to assure correct behaviour
         super().__init__(name=name, **kwargs)
         if not params:
             raise ValueError("`params` must be provided.")
+        self._original_params = params
 
         # The following dictionary will be used to create the appropriate
         # lambda function arguments:
@@ -140,8 +154,8 @@ class DynamicParam(param.Parameterized):
         # NOTE: unfortunately, parameters from the param library do not
         # provide a keyword argument to set a formatter for the slider's tick
         # value. As a workaround, the user can provide a formatter for each
-        # parameter, which will be stored in the following dictionary and later
-        # used in the instantiation of the widgets.
+        # parameter, which will be stored in the following dictionary and
+        # later used in the instantiation of the widgets.
         self.formatters = {}
 
         # create and attach the params to the class
@@ -209,10 +223,10 @@ class PanelLayout:
                 force a recompute.
         """
         # NOTE: More often than not, the numerical evaluation is going to be
-        # resource-intensive. By default, panel's sliders will force a recompute
-        # at every step change. As a consequence, the user experience will be
-        # laggy. To solve this problem, the update must be triggered on mouse-up
-        # event, which is set using throttled=True.
+        # resource-intensive. By default, panel's sliders will force a
+        # recompute at every step change. As a consequence, the user
+        # experience will be laggy. To solve this problem, the update must
+        # be triggered on mouse-up event, which is set using throttled=True.
         #
         # https://panel.holoviz.org/reference/panes/Param.html#disabling-continuous-updates-for-slider-widgets
 
@@ -225,21 +239,23 @@ class PanelLayout:
             )
             layout = "tb"
         self._layout = layout
+        self._ncols = ncols
+        self._throttled = throttled
 
-        # NOTE: here I create a temporary panel.Param object in order to reuse
-        # the code from the pn.Param.widget method, which returns the correct
-        # widget associated to a given parameter's type.
-        # Alternatively, I would need to copy parts of that method in order to
-        # deal with the different representations of param.Integer and
+        # NOTE: here I create a temporary panel.Param object in order to
+        # reuse the code from the pn.Param.widget method, which returns the
+        # correct widget associated to a given parameter's type.
+        # Alternatively, I would need to copy parts of that method in order
+        # to deal with the different representations of param.Integer and
         # param.Number depending if the bounds are None values.
-        # Note that I'm only interested in the widget type: panel is then going
-        # to recreate the widgets and setting the proper throttled value. This
-        # is definitely not an optimal procedure, as we are creating the "same"
-        # widget two times, but it works :)
+        # Note that I'm only interested in the widget type: panel is then
+        # going to recreate the widgets and setting the proper throttled
+        # value. This is definitely not an optimal procedure, as we are
+        # creating the "same" widget two times, but it works :)
         tmp_panel = pn.Param(self)
         widgets = dict()
         for k, v in self.mapping.items():
-            widgets[v] = { "type": type(tmp_panel.widget(v)) }
+            widgets[v] = {"type": type(tmp_panel.widget(v))}
             t = getattr(self.param, v)
             if isinstance(t, param.Number):
                 widgets[v]["throttled"] = throttled
@@ -307,8 +323,9 @@ class InteractivePlot(DynamicParam, PanelLayout):
             name : str
                 Unused parameter
             params : dict
-                In the keys there will be the symbols, in the values there will
-                be parameters to create the slider associated to a symbol.
+                In the keys there will be the symbols, in the values there
+                will be parameters to create the slider associated to
+                a symbol.
             kwargs : dict
                 Usual keyword arguments to be used by the backends and series.
         """
@@ -316,7 +333,7 @@ class InteractivePlot(DynamicParam, PanelLayout):
         layout = kwargs.pop("layout", "tb")
         ncols = kwargs.pop("ncols", 2)
         throttled = kwargs.pop("throttled", False)
-        use_latex = kwargs.pop("use_latex", True)
+        use_latex = kwargs.pop("use_latex", cfg["interactive"]["use_latex"])
 
         args = list(map(_plot_sympify, args))
         super().__init__(*args, name=name, params=params, use_latex=use_latex)
@@ -327,6 +344,7 @@ class InteractivePlot(DynamicParam, PanelLayout):
         is_3D = all([s.is_3D for s in series])
         # create the plot
         Backend = kwargs.pop("backend", THREE_D_B if is_3D else TWO_D_B)
+        kwargs["is_iplot"] = True
         self._backend = Backend(*series, **kwargs)
 
     def _create_series(self, *args, **kwargs):
@@ -345,23 +363,23 @@ class InteractivePlot(DynamicParam, PanelLayout):
                     *a, matrices=False, fill_ranges=False
                 )
                 new_args.append(Tuple(exprs[0], *ranges, label, sympify=False))
-                # new_args.append(Tuple(exprs[0], ranges[0], label, sympify=False))
             series = _build_complex_series(*new_args, interactive=True, **kwargs)
         elif is_vector:
             args = _preprocess(*args, matrices=False, fill_ranges=False)
             series = _build_vector_series(*args, interactive=True, **kwargs)
         else:
             for a in args:
-                # with interactive-parametric plots, vectors could have more free
-                # symbols than the number of dimensions. We set fill_ranges=False
-                # in order to not fill ranges, otherwise ranges will be created also
-                # for parameters. This means the user must provided all the necessary
-                # ranges.
+                # with interactive-parametric plots, vectors could have more
+                # free symbols than the number of dimensions. We set
+                # fill_ranges=False in order to not fill ranges, otherwise
+                # ranges will be created also for parameters. This means
+                # the user must provided all the necessary ranges.
                 exprs, ranges, label = _unpack_args(
                     *a, matrices=True, fill_ranges=False
                 )
                 if isinstance(_slice, (tuple, list)):
-                    # Sliced 3D vector field: each slice creates a unique series
+                    # Sliced 3D vector field: each slice creates a
+                    # unique series
                     kwargs2 = kwargs.copy()
                     kwargs2.pop("slice")
                     for s in _slice:
@@ -378,6 +396,61 @@ class InteractivePlot(DynamicParam, PanelLayout):
         """Return the plot object"""
         return self._backend.fig
 
+    @property
+    def backend(self):
+        """Return the backend"""
+        return self._backend
+
+    def save(self, *args, **kwargs):
+        """Save the current figure.
+        This is a wrapper to the backend's `save` function. Refer to the
+        backend's documentation to learn more about arguments and keyword
+        arguments.
+        """
+        self._backend.save(*args, **kwargs)
+
+    def __add__(self, other):
+        return self._do_sum(other)
+
+    def __radd__(self, other):
+        return other._do_sum(self)
+
+    def _do_sum(self, other):
+        """Differently from Plot.extend, this method creates a new plot object,
+        which uses the series of both plots and merges the _kwargs dictionary
+        of `self` with the one of `other`.
+        """
+        from spb.backends.base_backend import Plot
+        mergedeep = import_module('mergedeep')
+        merge = mergedeep.merge
+
+        if not isinstance(other, (Plot, InteractivePlot)):
+            raise TypeError(
+                "Both sides of the `+` operator must be instances of the "
+                "InteractivePlot or Plot class.\n"
+                "Received: {} + {}".format(type(self), type(other)))
+
+        series = self._backend.series
+        if isinstance(other, Plot):
+            series.extend(other.series)
+        else:
+            series.extend(other._backend.series)
+
+        backend_kw = self._backend._copy_kwargs()
+        panel_kw = {
+            "backend": type(self._backend),
+            "layout": self._layout,
+            "ncols": self._ncols,
+            "throttled": self._throttled,
+            "use_latex": self._use_latex,
+            "params": self._original_params,
+            "show": False
+        }
+
+        new_iplot = type(self)(**merge({}, backend_kw, panel_kw))
+        new_iplot._backend.series.extend(series)
+        return new_iplot
+
 
 def iplot(*args, show=True, **kwargs):
     """
@@ -391,13 +464,13 @@ def iplot(*args, show=True, **kwargs):
         expression, the tuple should have the following forms:
 
         1. line:
-           ``(expr, range, label [optional])``
+           `(expr, range, label [optional])`
         2. parametric line:
-           ``(expr1, expr2, expr3 [optional], range, label [optional])``
+           `(expr1, expr2, expr3 [optional], range, label [optional])`
         3. surface:
-           ``(expr, range1, range2, label [optional])``
+           `(expr, range1, range2, label [optional])`
         4. parametric surface:
-           ``(expr1, expr2, expr3, range1, range2, label [optional])``
+           `(expr1, expr2, expr3, range1, range2, label [optional])`
 
         The label is always optional, whereas the ranges must always be
         specified. The ranges will create the discretized domain.
@@ -405,9 +478,9 @@ def iplot(*args, show=True, **kwargs):
     params : dict
         A dictionary mapping the symbols to a parameter. The parameter can be:
 
-        1. an instance of ``param.parameterized.Parameter``.
+        1. an instance of `param.parameterized.Parameter`.
         2. a tuple of the form:
-           ``(default, min, max, N, tick_format, label, spacing)``
+           `(default, min, max, N, tick_format, label, spacing)`
            where:
 
            - default, min, max : float
@@ -417,36 +490,39 @@ def iplot(*args, show=True, **kwargs):
                 Number of steps of the slider.
            - tick_format : TickFormatter or None, optional
                 Provide a formatter for the tick value of the slider. If None,
-                ``panel`` will automatically apply a default formatter.
+                `panel` will automatically apply a default formatter.
                 Alternatively, an instance of
-                ``bokeh.models.formatters.TickFormatter`` can be used.
+                `bokeh.models.formatters.TickFormatter` can be used.
                 Default to None.
            - label: str, optional
                 Custom text associated to the slider.
            - spacing : str, optional
-                Specify the discretization spacing. Default to ``"linear"``, can
-                be changed to ``"log"``.
+                Specify the discretization spacing. Default to `"linear"`, can
+                be changed to `"log"`.
 
         Note that the parameters cannot be linked together (ie, one parameter
         cannot depend on another one).
 
+    aspect : (float, float) or str, optional
+        Set the aspect ratio of the plot. The value depends on the backend
+        being used. Read that backend's documentation to find out the
+        possible values.
+
     backend : Plot, optional
         The backend to be used to generate the plot. It must be a subclass of
-        ``spb.backends.base_backend.Plot``. If not provided, the module will
-        use the default backend. If ``MatplotlibBackend`` is used,
-        we must run the command ``%matplotlib widget`` at the start of the
-        notebook, otherwise the plot will not update.
+        `spb.backends.base_backend.Plot`. If not provided, the module will
+        use the default backend.
 
     layout : str, optional
         The layout for the controls/plot. Possible values:
 
-        - ``'tb'``: controls in the top bar.
-        - ``'bb'``: controls in the bottom bar.
-        - ``'sbl'``: controls in the left side bar.
-        - ``'sbr'``: controls in the right side bar.
+        - `'tb'`: controls in the top bar.
+        - `'bb'`: controls in the bottom bar.
+        - `'sbl'`: controls in the left side bar.
+        - `'sbr'`: controls in the right side bar.
 
-        Default layout to ``'tb'``. Note that side bar layouts may not
-        work well with some backends, and with ``MatplotlibBackend`` the widgets
+        Default layout to `'tb'`. Note that side bar layouts may not
+        work well with some backends, and with `MatplotlibBackend` the widgets
         are always going to be displayed below the figure.
 
     ncols : int, optional
@@ -466,8 +542,8 @@ def iplot(*args, show=True, **kwargs):
         Default to True.
         If True, it will return an object that will be rendered on the
         output cell of a Jupyter Notebook. If False, it returns an instance
-        of ``InteractivePlot``, which can later be be shown by calling the
-        ``show()`` method.
+        of `InteractivePlot`, which can later be be shown by calling the
+        `show()` method.
 
     use_latex : bool, optional
         Default to True.
@@ -476,13 +552,13 @@ def iplot(*args, show=True, **kwargs):
         representation will be used instead.
 
     detect_poles : boolean
-            Chose whether to detect and correctly plot poles.
-            Defaulto to False. To improve detection, increase the number of
-            discretization points and/or change the value of `eps`.
+        Chose whether to detect and correctly plot poles in line plots.
+        Defaulto to `False`. To improve detection, increase the number of
+        discretization points `n` and/or change the value of `eps`.
 
     eps : float
         An arbitrary small value used by the `detect_poles` algorithm.
-        Default value to 0.1. Before changing this value, it is better to
+        Default value to 0.1. Before changing this value, it is recommended to
         increase the number of discretization points.
 
     n1, n2, n3 : int, optional
@@ -491,23 +567,22 @@ def iplot(*args, show=True, **kwargs):
 
     n : int, optional
         Set the number of discretization points on all directions.
-        It overrides ``n1, n2, n3``.
+        It overrides `n1, n2, n3`.
 
     nc : int, optional
         Number of discretization points for the contour plot when
-        ``is_complex=True``.
+        `is_vector=True`.
 
     polar : boolean, optional
-        Default to False. If True, generate a polar plot of a curve with radius
-        `expr` as a function of the range
+        Default to False. If True, generate a polar plot of a curve with
+        radius `expr` as a function of the range
 
     throttled : boolean, optional
         Default to False. If True the recompute will be done at mouse-up event
         on sliders. If False, every slider tick will force a recompute.
 
     title : str, optional
-        Title of the plot. It is set to the latex representation of
-        the expression, if the plot has only one expression.
+        Title of the plot.
 
     xlabel : str, optional
         Label for the x-axis.
@@ -519,13 +594,13 @@ def iplot(*args, show=True, **kwargs):
         Label for the z-axis.
 
     xlim : (float, float), optional
-        Denotes the x-axis limits, ``(min, max)``.
+        Denotes the x-axis limits, `(min, max)`.
 
     ylim : (float, float), optional
-        Denotes the y-axis limits, ``(min, max)``.
+        Denotes the y-axis limits, `(min, max)`.
 
     zlim : (float, float), optional
-        Denotes the z-axis limits, ``(min, max)``.
+        Denotes the z-axis limits, `(min, max)`.
 
 
     Examples
@@ -533,34 +608,38 @@ def iplot(*args, show=True, **kwargs):
 
     .. jupyter-execute::
 
-       >>> from sympy import (symbols, sqrt, cos, exp, sin, pi,
-       ...     Matrix, Plane, Polygon, I, log)
-       >>> from spb.interactive import iplot
-       >>> from spb.backends.matplotlib import MB
-       >>> x, y, z = symbols("x, y, z")
+       from sympy import (symbols, sqrt, cos, exp, sin, pi, re, im,
+           Matrix, Plane, Polygon, I, log)
+       from spb.interactive import iplot
+       from spb.backends.matplotlib import MB
+       from spb.backends.bokeh import BB
+       from spb.backends.plotly import PB
+       x, y, z = symbols("x, y, z")
 
-    Surface plot between -10 <= x, y <= 10 with a damping parameter varying from
-    0 to 1, with a default value of 0.15, discretized with 100 points on both
-    directions. Note the use of ``threed=True`` to specify a 3D plot. If
-    ``threed=False``, a contour plot will be generated.
+    Surface plot between -10 <= x, y <= 10 with a damping parameter varying
+    from 0 to 1, with a default value of 0.15, discretized with 100 points
+    on both directions. Note the use of `threed=True` to specify a 3D plot.
+    If `threed=False`, a contour plot will be generated.
 
     .. jupyter-execute::
 
-       >>> r = sqrt(x**2 + y**2)
-       >>> d = symbols('d')
-       >>> expr = 10 * cos(r) * exp(-r * d)
-       >>> iplot(
-       ...     (expr, (x, -10, 10), (y, -10, 10)),
-       ...     params = { d: (0.15, 0, 1) },
-       ...     title = "My Title",
-       ...     xlabel = "x axis",
-       ...     ylabel = "y axis",
-       ...     zlabel = "z axis",
-       ...     backend = MB,
-       ...     n = 100,
-       ...     threed = True)
+       r = sqrt(x**2 + y**2)
+       d = symbols('d')
+       expr = 10 * cos(r) * exp(-r * d)
+       iplot(
+           (expr, (x, -10, 10), (y, -10, 10)),
+           params = { d: (0.15, 0, 1) },
+           title = "My Title",
+           xlabel = "x axis",
+           ylabel = "y axis",
+           zlabel = "z axis",
+           backend = PB,
+           n = 50,
+           threed = True
+       )
 
-    A line plot illustrating the use of multiple expressions and:
+    A line plot of the magnitude of a transfer function, illustrating the use
+    of multiple expressions and:
 
     1. some expression may not use all the parameters.
     2. custom labeling of the expressions.
@@ -571,81 +650,140 @@ def iplot(*args, show=True, **kwargs):
 
     .. jupyter-execute::
 
-       >>> from bokeh.models.formatters import PrintfTickFormatter
-       >>> formatter = PrintfTickFormatter(format="%.3f")
-       >>> A1, A2, k = symbols("A1, A2, k")
-       >>> iplot(
-       ...     (log(x) + A1 * sin(k * x), (x, 1e-05, 20), "f1"),
-       ...     (exp(-(x - 2)) + A2 * cos(x), (x, 0, 20), "f2"),
-       ...     (10 + 5 * cos(k * x), A2 * 25 * sin(x), (x, 0, pi)),
-       ...     params = {
-       ...         k: (1, 0, 5),
-       ...         A1: (0.05, 0, 1, 20, None, "Ampl 1"),
-       ...         A2: (0.2, 0, 1, 200, formatter, "Ampl 2"),
-       ...     },
-       ...     backend = MB,
-       ...     ylim = (-4, 10))
+       from bokeh.models.formatters import PrintfTickFormatter
+       formatter = PrintfTickFormatter(format="%.3f")
+       kp, t, z, o = symbols("k_P, tau, zeta, omega")
+       G = kp / (I**2 * t**2 * o**2 + 2 * z * t * o * I + 1)
+       mod = lambda x: 20 * log(sqrt(re(x)**2 + im(x)**2), 10)
+       iplot(
+           (mod(G.subs(z, 0)), (o, 0.1, 100), "G(z=0)"),
+           (mod(G.subs(z, 1)), (o, 0.1, 100), "G(z=1)"),
+           (mod(G), (o, 0.1, 100), "G"),
+           params = {
+               kp: (1, 0, 3),
+               t: (1, 0, 3),
+               z: (0.2, 0, 1, 200, formatter, "z")
+           },
+           backend = BB,
+           n = 2000,
+           xscale = "log",
+           xlabel = "Frequency, omega, [rad/s]",
+           ylabel = "Magnitude [dB]",
+           use_latex = False
+       )
 
     A 3D slice-vector plot. Note: whenever we want to create parametric vector
-    plots, we should set ``is_vector=True``.
+    plots, we should set `is_vector=True`.
 
     .. jupyter-execute::
 
-       >>> a, b = symbols("a, b")
-       >>> iplot(
-       ...     (Matrix([z * a, y * b, x]), (x, -5, 5), (y, -5, 5), (z, -5, 5)),
-       ...     params = {
-       ...         a: (1, 0, 5),
-       ...         b: (1, 0, 5)
-       ...     },
-       ...     backend = MB,
-       ...     n = 10,
-       ...     is_vector = True,
-       ...     quiver_kw = {"length": 0.15},
-       ...     slice = Plane((0, 0, 0), (0, 1, 0)))
+       a, b = symbols("a, b")
+       iplot(
+           (Matrix([z * a, y * b, x]), (x, -5, 5), (y, -5, 5), (z, -5, 5)),
+           params = {
+               a: (1, 0, 5),
+               b: (1, 0, 5)
+           },
+           backend = PB,
+           is_vector = True,
+           n = 10,
+           slice = Plane((0, 0, 0), (0, 1, 0)),
+           quiver_kw = {"sizeref": 8},
+           use_latex = False
+       )
 
     A parametric complex domain coloring plot. Note: whenever we want to create
-    parametric complex plots, we must set ``is_complex=True``.
+    parametric complex plots, we must set `is_complex=True`.
 
     .. jupyter-execute::
 
-        >>> iplot(
-        ...     ((z**2 + 1) / (x * (z**2 - 1)), (z, -4 - 2 * I, 4 + 2 * I)),
-        ...     params = {
-        ...         x: (1, -2, 2)
-        ...     },
-        ...     backend = MB,
-        ...     is_complex = True,
-        ...     coloring = "b")
+       iplot(
+           ((z**2 + 1) / (x * (z**2 - 1)), (z, -4 - 2 * I, 4 + 2 * I)),
+           params = {
+               x: (1, -2, 2)
+           },
+           backend = MB,
+           is_complex = True,
+           coloring = "b",
+           grid = False,
+           use_latex = False
+       )
 
 
-    A parametric plot of a symbolic polygon. Note the use of ``param`` to create
+    A parametric plot of a symbolic polygon. Note the use of `param` to create
     an integer slider.
 
     .. jupyter-execute::
 
-       >>> import param
-       >>> a, b, c, d = symbols('a:d')
-       >>> iplot(
-       ...     (Polygon((a, b), c, n=d), ),
-       ...     params = {
-       ...         a: (0, -2, 2),
-       ...         b: (0, -2, 2),
-       ...         c: (1, 0, 5),
-       ...         d: param.Integer(3, softbounds=(3, 10), label="n"),
-       ...     },
-       ...     backend = MB,
-       ...     fill = False,
-       ...     aspect = "equal",
-       ...     use_latex = False)
+       import param
+       a, b, c, d = symbols('a:d')
+       iplot(
+           (Polygon((a, b), c, n=d), ),
+           params = {
+               a: (0, -2, 2),
+               b: (0, -2, 2),
+               c: (1, 0, 5),
+               d: param.Integer(3, softbounds=(3, 10), label="n"),
+           },
+           backend = BB,
+           is_filled = False,
+           aspect = "equal",
+           use_latex = False
+       )
 
-    Serves the interactive plot on a separate browser window. Note: only
-    ``BokehBackend`` and ``PlotlyBackend`` are supported for this operation
-    mode.
+    Combine together `InteractivePlot` and ``Plot`` instances. The same
+    parameters dictionary must be used for every ``iplot`` command. Note:
+
+    1. the first plot dictates the labels, title and wheter to show the legend
+       or not.
+    2. Instances of ``Plot`` class must be place on the right side of the `+`
+       sign.
+    3. `show=False` has been set in order for ``iplot`` to return an instance
+       of ``InteractivePlot``, which supports addition.
+
+    .. code-block:: python
+
+       from spb.functions import plot
+       u = symbols("u")
+       params = {
+           u: (1, 0, 2)
+       }
+       p1 = iplot(
+           (cos(u * x), (x, -5, 5)),
+           params = params,
+           backend = MB,
+           xlabel = "x1",
+           ylabel = "y1",
+           title = "title 1",
+           legend = True,
+           show = False
+       )
+       p2 = iplot(
+           (sin(u * x), (x, -5, 5)),
+           params = params,
+           backend = MB,
+           xlabel = "x2",
+           ylabel = "y2",
+           title = "title 2",
+           show = False
+       )
+       p3 = plot(sin(x)*cos(x), (x, -5, 5), backend=MB,
+           adaptive=False, n=50,
+           is_point=True, is_filled=True,
+           line_kw=dict(marker="^"), show=False)
+       p = p1 + p2 + p3
+       p.show()
+
+    Serves the interactive plot on a separate browser window. Note:
+
+    1. only ``BokehBackend`` and ``PlotlyBackend`` are supported for this
+       operation mode.
+    2. the output of ``iplot`` is captures into a variable.
+    3. `show=True` has been set in order for ``iplot`` to return a `panel`
+       object.
 
     .. code-block:: python
 
-       from sympy import *
        from spb.backends.bokeh import BB
        from bokeh.models.formatters import PrintfTickFormatter
        formatter = PrintfTickFormatter(format='%.4f')
@@ -657,7 +795,7 @@ def iplot(*args, show=True, **kwargs):
 
        t = iplot(
            (r1, (t, 0, 2*pi)),
-           params = { 
+           params = {
                p1: (0.035, -0.035, 0.035, 50, formatter),
                p2: (0.005, -0.02, 0.02, 50, formatter),
                r: (2, 2, 5, 3),
@@ -678,35 +816,72 @@ def iplot(*args, show=True, **kwargs):
     =====
 
     1. This function is specifically designed to work within Jupyter Notebook.
-       However, it is also possible to use it from a regular Python interpreter,
-       but only with ``BokehBackend`` and ``PlotlyBackend``. In such cases, we
-       have to call ``iplot(..., backend=BB).show()``, which will create a
-       server process loading the interactive plot on the browser.
+       However, it is also possible to use it from a regular Python,
+       interpreter but only with ``BokehBackend`` and ``PlotlyBackend``.
+       In such cases, it must be called with ``iplot(..., backend=BB).show()``, which will create a server process loading the interactive plot on
+       the browser.
 
     2. Some examples use an instance of ``PrintfTickFormatter`` to format the
-       value shown by a slider. This class is exposed by Bokeh, but can be used
-       by ``iplot`` with any backend. Refer to [#fn1]_ for more information
-       about tick formatting.
-    
-    3. We have seen the duality of the keyword argument ``show``:
+       value shown by a slider. This class is exposed by Bokeh, but can be
+       used by ``iplot`` with any backend. Refer to [#fn1]_ for more
+       information about tick formatting.
 
-       * If True, the function returns a ``panel`` object that will be rendered
+    3. The duality of the keyword argument ``show``:
+
+       * If True, the function returns a `panel` object that will be rendered
          on the output cell of a Jupyter Notebook.
-       * If False, it returns an instance of ``InteractivePlot``.
+       * If False, it returns an instance of `InteractivePlot`.
 
        Let's focus on the syntax ``t = iplot(..., show=True, backend=BB)``, as
        shown in the last example.
-       Here, the variable ``t`` captures the ``panel`` object, thus nothing
+       Here, the variable `t` captures the `panel` object, thus nothing
        will be rendered on the output cell. We can use this variable to serve
        the interactive plot through a server process on a separate browser
        window, by calling ``t.show()``. In doing so, the overall interactive
        plot is not subjected to the width limitation of a classical Jupyter
        Notebook. It is possible to play with the following keyword arguments
        to further customize the look and take advantage of the full page:
-       ``size, ncols, layout``.
+       `size, ncols, layout`.
        As stated before, only ``BokehBackend`` and ``PlotlyBackend`` are
        supported in this mode of operation.
 
+    4. Say we are creating two different interactive plots and capturing
+       their output on two variables, using ``show=False``. For example,
+       ``p1 = iplot(..., show=False)`` and ``p2 = iplot(..., show=False)``.
+       Then, running ``p1.show()`` on the screen will result in an error.
+       This is standard behaviour that can't be changed, as `panel's`
+       parameters are class attributes that gets deleted each time a new
+       instance is created.
+
+    5. ``MatplotlibBackend`` can be used, but the resulting figure is just a
+       PNG image without any interactive frame. Thus, data exploration is not
+       great. Therefore, the use of ``PlotlyBackend`` or ``BokehBackend`` is
+       encouraged.
+
+    6. Once this module has been loaded, there could be problems with all
+       other plotting functions when using ``BokehBackend``, namely the
+       figure won't show up in the output cell. If that is the case, we need
+       to turn off  automatic updates on panning by setting
+       ``update_event=False`` in the function call.
+
+    7. When ``BokehBackend`` is used:
+
+       * the user-defined theme won't be applied.
+       * rendering of gradient lines is slow.
+       * color bars might not update their ranges.
+
+    8. Once this module has been loaded and ``iplot`` has been executed, if
+       the kernel of the notebook needs to be restarted, the safest procedure
+       is the following:
+
+       * save the current notebook.
+       * close the notebook and Jupyter server.
+       * restart Jupyter server and open the notebook.
+       * reload the cells.
+
+       Failing to follow this procedure might results in the notebook to
+       become  unresponsive once the module has been reloaded, with several
+       errors appearing on the output cell.
 
 
     References
@@ -730,10 +905,10 @@ def create_widgets(params, **kwargs):
     params : dict
         A dictionary mapping the symbols to a parameter. The parameter can be:
 
-        1. an instance of ``param.parameterized.Parameter``. Refer to [#fn2]_
+        1. an instance of `param.parameterized.Parameter`. Refer to [#fn2]_
            for a list of available parameters.
         2. a tuple of the form:
-           ``(default, min, max, N, tick_format, label, spacing)``
+           `(default, min, max, N, tick_format, label, spacing)`
            where:
 
            - default, min, max : float
@@ -743,15 +918,15 @@ def create_widgets(params, **kwargs):
                 Number of steps of the slider.
            - tick_format : TickFormatter or None, optional
                 Provide a formatter for the tick value of the slider. If None,
-                ``panel`` will automatically apply a default formatter.
+                `panel` will automatically apply a default formatter.
                 Alternatively, an instance of
-                ``bokeh.models.formatters.TickFormatter`` can be used.
+                `bokeh.models.formatters.TickFormatter` can be used.
                 Default to None.
            - label: str, optional
                 Custom text associated to the slider.
            - spacing : str, optional
-                Specify the discretization spacing. Default to ``"linear"``, can
-                be changed to ``"log"``.
+                Specify the discretization spacing. Default to `"linear"`,
+                can be changed to `"log"`.
 
         Note that the parameters cannot be linked together (ie, one parameter
         cannot depend on another one).
@@ -767,22 +942,25 @@ def create_widgets(params, **kwargs):
     =======
 
     widgets : dict
-        A dictionary mapping the symbols from ``params`` to the appropriate
+        A dictionary mapping the symbols from `params` to the appropriate
         widget.
 
 
     Examples
     ========
 
-    >>> from sympy.abc import x, y, z
-    >>> from spb.interactive import create_widgets
-    >>> from bokeh.models.formatters import PrintfTickFormatter
-    >>> formatter = PrintfTickFormatter(format="%.4f")
-    >>> r = create_widgets({
-    ...     x: (0.035, -0.035, 0.035, 100, formatter),
-    ...     y: (200, 1, 1000, 10, None, "test", "log"),
-    ...     z: param.Integer(3, softbounds=(3, 10), label="n")
-    ... })
+    .. jupyter-execute::
+
+       from sympy.abc import x, y, z
+       from spb.interactive import create_widgets
+       import param
+       from bokeh.models.formatters import PrintfTickFormatter
+       formatter = PrintfTickFormatter(format="%.4f")
+       r = create_widgets({
+           x: (0.035, -0.035, 0.035, 100, formatter),
+           y: (200, 1, 1000, 10, None, "test", "log"),
+           z: param.Integer(3, softbounds=(3, 10), label="n")
+       })
 
 
     References
